'use client';

import {
	Button,
	Title1,
	makeStyles,
	tokens,
	Tooltip,
	Input,
	SplitButton,
	Body1Stronger,
	Caption1,
	shorthands,
	Menu,
	MenuItem,
	MenuList,
	MenuPopover,
	MenuTrigger,
	Body1,
} from '@fluentui/react-components';
import {
	ArrowSyncRegular,
	SearchRegular,
	AppsListFilled,
	StarFilled,
	EarthRegular,
	EarthLeafRegular,
	GlobeRegular,
	MoreHorizontalFilled,
	ArrowCircleUpRegular,
	Globe24Regular,
	ArrowSync24Regular,
	Search24Regular,
	PersonCircle24Regular,
	QuestionCircle24Regular,
	Star24Filled,
	Star16Filled,
} from '@fluentui/react-icons';

import Image from 'next/image';

import {
	FolderRegular,
	EditRegular,
	OpenRegular,
	DocumentRegular,
	PeopleRegular,
	DocumentPdfRegular,
	VideoRegular,
} from '@fluentui/react-icons';
import {
	PresenceBadgeStatus,
	Avatar,
	DataGridBody,
	DataGridRow,
	DataGrid,
	DataGridHeader,
	DataGridHeaderCell,
	DataGridCell,
	TableCellLayout,
	TableColumnDefinition,
	createTableColumn,
} from '@fluentui/react-components';
import { ArrowCircleUpFilled } from '@fluentui/react-icons/lib/fonts';

type FileCell = {
	label: string;
	path: string;
};

type LastUpdatedCell = {
	label: string;
	timestamp: number;
};

enum ProjectType {
	World = 'World',
	Avatar = 'Avatar',
	Unknown = 'Unknown',
}

type Item = {
	stared: boolean;
	file: FileCell;
	projectType: ProjectType;
	unity: string;
	lastUpdated: LastUpdatedCell;
};

const items: Item[] = [
	{
		stared: false,
		file: { label: 'My home', path: 'G:\\Unity\\My home' },
		projectType: ProjectType.World,
		unity: '2020.3.14f1',
		lastUpdated: { label: '7h ago', timestamp: 1 },
	},
	{
		stared: false,
		file: { label: 'Mint-Usually', path: 'G:\\Unity\\Mint-Usually' },
		projectType: ProjectType.Avatar,
		unity: '2020.3.14f1',
		lastUpdated: { label: 'Yesterday at 1:45 PM', timestamp: 2 },
	},
];

const columns: TableColumnDefinition<Item>[] = [
	createTableColumn<Item>({
		columnId: 'star',
		compare: (a, b) => {
			return a.stared === b.stared ? 0 : a.stared ? -1 : 1;
		},
		renderHeaderCell: () => {
			return <Star16Filled />;
		},
		renderCell: item => {
			return (
				<TableCellLayout truncate>
					<Button icon={<Star24Filled />} appearance="transparent" />
				</TableCellLayout>
			);
		},
	}),
	createTableColumn<Item>({
		columnId: 'projectName',
		compare: (a, b) => {
			return a.file.label.localeCompare(b.file.label);
		},
		renderHeaderCell: () => {
			return <Body1Stronger>Project Name</Body1Stronger>;
		},
		renderCell: item => {
			return (
				<TableCellLayout truncate>
					<div className="flex flex-col">
						<Body1Stronger>{item.file.label}</Body1Stronger>
						<Caption1>{item.file.path}</Caption1>
					</div>
				</TableCellLayout>
			);
		},
	}),
	createTableColumn<Item>({
		columnId: 'projectType',
		renderHeaderCell: () => {
			return <Body1Stronger>Project Type</Body1Stronger>;
		},
		renderCell: item => {
			return (
				// eslint-disable-next-line react/jsx-no-undef
				<TableCellLayout
					truncate
					media={
						item.projectType === ProjectType.World ? (
							<Globe24Regular />
						) : item.projectType === ProjectType.Avatar ? (
							<PersonCircle24Regular />
						) : (
							<QuestionCircle24Regular />
						)
					}
				>
					{item.projectType}
				</TableCellLayout>
			);
		},
	}),
	createTableColumn<Item>({
		columnId: 'unity',
		compare: (a, b) => {
			return a.unity.localeCompare(b.unity);
		},
		renderHeaderCell: () => {
			return <Body1Stronger>Unity</Body1Stronger>;
		},
		renderCell: item => {
			return (
				<TableCellLayout truncate>
					<Tooltip content="Unity Version Upgrade Available" relationship="description">
						<Button
							appearance="outline"
							icon={<ArrowCircleUpRegular color={tokens.colorPaletteGreenForeground1} />}
							color={tokens.colorPaletteGreenForeground1}
							style={{
								borderColor: tokens.colorPaletteGreenForeground1,
							}}
						>
							<Body1Stronger>{item.unity}</Body1Stronger>
						</Button>
					</Tooltip>
				</TableCellLayout>
			);
		},
	}),
	createTableColumn<Item>({
		columnId: 'lastModified',
		compare: (a, b) => {
			return a.lastUpdated.timestamp - b.lastUpdated.timestamp;
		},
		renderHeaderCell: () => {
			return <Body1Stronger truncate>Last Modified</Body1Stronger>;
		},

		renderCell: item => {
			return item.lastUpdated.label;
		},
	}),
	createTableColumn<Item>({
		columnId: 'buttons',
		renderHeaderCell: () => {
			return '';
		},
		renderCell: item => {
			return (
				<TableCellLayout className="flex">
					<Button appearance="primary">Open Project</Button>
					<Button appearance="outline">Manage Project</Button>
					<Menu>
						<MenuTrigger disableButtonEnhancement>
							<Button icon={<MoreHorizontalFilled />} appearance="subtle" />
						</MenuTrigger>

						<MenuPopover>
							<MenuList>
								<MenuItem>Open Project Folder</MenuItem>
								<MenuItem>Create Backup</MenuItem>
								<MenuItem>Remove Project</MenuItem>
							</MenuList>
						</MenuPopover>
					</Menu>
				</TableCellLayout>
			);
		},
	}),
];

const useStyles = makeStyles({
	root: {
		display: 'flex',
		flexDirection: 'column',
		justifyContent: 'center',
		paddingLeft: '1rem',
		paddingRight: '1rem',
		paddingTop: '0.5rem',
		paddingBottom: '1rem',
		backgroundColor: tokens.colorNeutralBackground2,
		width: '100%',
	},
	header: {
		display: 'flex',
		flexDirection: 'row',
		justifyContent: 'space-between',
		width: '100%',
		alignItems: 'flex-end',
	},
	title: {
		display: 'flex',
		flexDirection: 'row',
		alignItems: 'center',
		columnGap: '0.5rem',
	},
	search: {
		display: 'grid',
		gridTemplateColumns: '1fr repeat(2, auto)',
		columnGap: '0.5rem',
		width: '100%',
	},
	table: {
		marginTop: '1rem',
		height: '100%',
		backgroundColor: tokens.colorNeutralBackground1,
		...shorthands.borderRadius('4px'),
	},
	dataGrid: {
		...shorthands.borderRadius('10px'),
	},
	unityUpdateButton: {
		color: tokens.colorPaletteGreenForeground1,
	},
	gridHeader: {
		width: '50px',
		':hover': {
			backgroundColor: 'transparent',
		},
	},
});

export default function Home() {
	const classes = useStyles();

	return (
		<main className={classes.root}>
			<div className={classes.header}>
				<div className={classes.title}>
					<Title1>Projects</Title1>
					<Tooltip content="Refresh Projects" relationship="description">
						<Button appearance="subtle" icon={<ArrowSync24Regular />} aria-description="Refresh Projects" />
					</Tooltip>
				</div>
				<div className={classes.search}>
					<Input appearance="filled-lighter" placeholder="Search Projects" contentBefore={<Search24Regular />}></Input>
					<Button appearance="subtle" icon={<AppsListFilled />}>
						List View
					</Button>
					<SplitButton appearance="primary" size="medium">
						Create New Project
					</SplitButton>
				</div>
			</div>
			<div className={classes.table}>
				<DataGrid
					items={items}
					columns={columns}
					sortable
					getRowId={item => item.file.label}
					onSelectionChange={(e, data) => console.log(data)}
					className={classes.dataGrid}
				>
					<DataGridHeader>
						<DataGridRow>
							{({ renderHeaderCell }) => <DataGridHeaderCell>{renderHeaderCell()}</DataGridHeaderCell>}
						</DataGridRow>
					</DataGridHeader>
					<DataGridBody<Item>>
						{({ item, rowId }) => (
							<DataGridRow<Item> key={rowId}>
								{({ renderCell }) => <DataGridCell>{renderCell(item)}</DataGridCell>}
							</DataGridRow>
						)}
					</DataGridBody>
				</DataGrid>
			</div>
		</main>
	);
}
